==> demoniac-0.3.tar.gz: demoniac-0.3/libfx/menu.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Menu managment for calodox DemoNiac.
 * leto/calodox, 8 2001.
 * (leto@calodox.org)
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "menu.h"
#include "glmain.h"
#include "pixmaps.h"
#include "about.h"

// functions declarations
static gint cbLoadProject(GtkWidget *widget, gpointer data);
static gint cbSaveProject(GtkWidget *widget, gpointer data);
void menuCbNew(GtkWidget *widget, gpointer data);
void menuCbOpen(GtkWidget *widget, gpointer data);
void menuCbSave(GtkWidget *widget, gpointer data);
void menuCbSaveAs(GtkWidget *widget, gpointer data);

// called by the OK button of the open file selection dialog.
static gint cbLoadProject(GtkWidget *widget, gpointer data) {
	Loader l;
	char *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));

	if (l.loadFile(filename)) {
		glMainptr->load(&l);
		printf("Loaded %s.\n", glMainptr->filename);
	} else {
	}
	gtk_widget_destroy(GTK_WIDGET(data));
	return TRUE;
}

// called by the save file selection dialog
static gint cbSaveProject(GtkWidget *widget, gpointer data) {
	Saver s;
	char *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));

	if (s.saveFile(filename)) {
		glMainptr->save(&s);
	} else {
		glMainptr->filename = 0;
	}

	gtk_widget_destroy(GTK_WIDGET(data));
	return TRUE;
}


// called by the file/new menu
void menuCbNew(GtkWidget *widget, gpointer data) {
	glMainptr->clearAll();
}

// window show/hide callback
void menuWindowShowHide(GtkWidget *widget, Dlg *dlg) {
	dlg->switchVisibility();
}

// called by the file/open menu
void menuCbOpen(GtkWidget *widget, gpointer data) {

	GtkWidget *filew;
	filew = gtk_file_selection_new ("Open Project");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) cbLoadProject, filew );

	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
				(filew)->cancel_button),
			"clicked", (GtkSignalFunc) gtk_widget_destroy,
			GTK_OBJECT (filew));
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION (filew));
	gtk_widget_show(filew);

}

// called by the file/Save menu
void menuCbSave(GtkWidget *widget, gpointer data) {
	Saver s;

	if (glMainptr->filename) {
	
		if (s.saveFile(glMainptr->filename)) {
			glMainptr->save(&s);
			printf("saved to %s\n", glMainptr->filename);
		} else {
			perror(glMainptr->filename);
		}
	} else {
		menuCbSaveAs(widget, data);
	}
}


// called by the file/Save As menu
void menuCbSaveAs(GtkWidget *widget, gpointer data) {
	GtkWidget *filew;
	filew = gtk_file_selection_new ("Save Project As");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) cbSaveProject, filew );

	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
				(filew)->cancel_button),
			"clicked", (GtkSignalFunc) gtk_widget_destroy,
			GTK_OBJECT (filew));

	if (glMainptr->filename)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), glMainptr->filename);

	gtk_widget_show(filew);
}

//! called when the menu help/about is activated
void menuCbAbout(GtkWidget *widget, gpointer data) {
	static GtkWidget *about = 0;

	if (!about)
		about = create_about();

	gtk_widget_show(about);
}

/* This is the GtkItemFactoryEntry structure used to generate new menus.
   Item 1: The menu path. The letter after the underscore indicates an
   accelerator key once the menu is open.
   Item 2: The accelerator key for the entry
   Item 3: The callback function.
   Item 4: The callback action.  This changes the parameters with
   which the function is called.  The default is 0.
   Item 5: The item type, used to define what kind of an item it is.
   Here are the possible values:

   NULL               -> "<Item>"
   ""                 -> "<Item>"
   "<Title>"          -> create a title item
   "<Item>"           -> create a simple item
   "<CheckItem>"      -> create a check item
   "<ToggleItem>"     -> create a toggle item
   "<RadioItem>"      -> create a radio item
   <path>             -> path of a radio item to link against
   "<Separator>"      -> create a separator
   "<Branch>"         -> create an item to hold sub items (optional)
   "<LastBranch>"     -> create a right justified branch 
 */

static GtkItemFactoryEntry menu_items[] = {
        { "/_File",         NULL,         NULL, 0, "<Branch>" },
        { "/File/_New",    "<control>N", (GtkSignalFunc) menuCbNew, 0, NULL },
        { "/File/_Open",    "<control>O", (GtkSignalFunc) menuCbOpen, 0, NULL },
        { "/File/_Save",    "<control>S", (GtkSignalFunc) menuCbSave, 0, NULL },
        { "/File/Save _As", NULL,         (GtkSignalFunc) menuCbSaveAs, 0, NULL },
        { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
        { "/File/Quit",     "<control>Q", (GtkSignalFunc) cbQuit, 0, NULL },
        { "/_Effects",      NULL,         NULL, 0, "<Branch>" },
        { "/Effects/_New",  NULL,         NULL, 0, "<Branch>"},
	{ "/Windows",	    NULL,	  NULL, 0, "<Branch>"},
        { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
        { "/_Help/About",   NULL,         (GtkSignalFunc) menuCbAbout, 0, NULL },
};

GtkItemFactory *setupMenu( GtkWidget  *window,
                GtkWidget **menubar )
{
        GtkItemFactory *item_factory;
        GtkAccelGroup *accel_group;
        gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

        accel_group = gtk_accel_group_new ();

        /* This function initializes the item factory.
           Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
           or GTK_TYPE_OPTION_MENU.
           Param 2: The path of the menu.
           Param 3: A pointer to a gtk_accel_group.  The item factory sets up
           the accelerator table while generating menus.
         */

        item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                        accel_group);

        /* This function generates the menu items. Pass the item factory,
           the number of items in the array, the array itself, and any
           callback data for the the menu items. */
        gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

        /* Attach the new accelerator group to the window. */
        gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

        if (menubar)
                /* Finally, return the actual menu bar created by the item factory. */
                *menubar = gtk_item_factory_get_widget (item_factory, "<main>");

	return item_factory;	
}

==> demoniac-0.4.tar.gz: demoniac-0.4/libfx/menu.cpp <==
/*
 * Calodox Demoniac
 * Copyright (C) 2001 Julien Pilet
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Menu managment for calodox DemoNiac.
 * leto/calodox, 8 2001.
 * (leto@calodox.org)
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "menu.h"
#include "glmain.h"
#include "pixmaps.h"
#include "about.h"

// functions declarations
static gint cbLoadProject(GtkWidget *widget, gpointer data);
static gint cbSaveProject(GtkWidget *widget, gpointer data);
void menuCbNew(GtkWidget *widget, gpointer data);
void menuCbOpen(GtkWidget *widget, gpointer data);
void menuCbSave(GtkWidget *widget, gpointer data);
void menuCbSaveAs(GtkWidget *widget, gpointer data);

// called by the OK button of the open file selection dialog.
static gint cbLoadProject(GtkWidget *widget, gpointer data) {
	Loader l;
	char *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));

	if (l.loadFile(filename)) {
		glMainptr->load(&l);
		printf("Loaded %s.\n", glMainptr->filename);
	} else {
	}
	gtk_widget_destroy(GTK_WIDGET(data));
	return TRUE;
}

// called by the save file selection dialog
static gint cbSaveProject(GtkWidget *widget, gpointer data) {
	Saver s;
	char *filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));

	if (s.saveFile(filename)) {
		glMainptr->save(&s);
	} else {
		glMainptr->filename = 0;
	}

	gtk_widget_destroy(GTK_WIDGET(data));
	return TRUE;
}


// called by the file/new menu
void menuCbNew(GtkWidget *widget, gpointer data) {
	glMainptr->clearAll();
}

// window show/hide callback
void menuWindowShowHide(GtkWidget *widget, Dlg *dlg) {
	dlg->switchVisibility();
}

// called by the file/open menu
void menuCbOpen(GtkWidget *widget, gpointer data) {

	GtkWidget *filew;
	filew = gtk_file_selection_new ("Open Project");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) cbLoadProject, filew );

	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
				(filew)->cancel_button),
			"clicked", (GtkSignalFunc) gtk_widget_destroy,
			GTK_OBJECT (filew));
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION (filew));
	gtk_widget_show(filew);

}

// called by the file/Save menu
void menuCbSave(GtkWidget *widget, gpointer data) {
	Saver s;

	if (glMainptr->filename) {
	
		if (s.saveFile(glMainptr->filename)) {
			glMainptr->save(&s);
			printf("saved to %s\n", glMainptr->filename);
		} else {
			perror(glMainptr->filename);
		}
	} else {
		menuCbSaveAs(widget, data);
	}
}


// called by the file/Save As menu
void menuCbSaveAs(GtkWidget *widget, gpointer data) {
	GtkWidget *filew;
	filew = gtk_file_selection_new ("Save Project As");
	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
			"clicked", (GtkSignalFunc) cbSaveProject, filew );

	/* Connect the cancel_button to destroy the widget */
	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
				(filew)->cancel_button),
			"clicked", (GtkSignalFunc) gtk_widget_destroy,
			GTK_OBJECT (filew));

	if (glMainptr->filename)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), glMainptr->filename);

	gtk_widget_show(filew);
}

//! called when the menu help/about is activated
void menuCbAbout(GtkWidget *widget, gpointer data) {
	static GtkWidget *about = 0;

	if (!about)
		about = create_about();

	gtk_widget_show(about);
}

/* This is the GtkItemFactoryEntry structure used to generate new menus.
   Item 1: The menu path. The letter after the underscore indicates an
   accelerator key once the menu is open.
   Item 2: The accelerator key for the entry
   Item 3: The callback function.
   Item 4: The callback action.  This changes the parameters with
   which the function is called.  The default is 0.
   Item 5: The item type, used to define what kind of an item it is.
   Here are the possible values:

   NULL               -> "<Item>"
   ""                 -> "<Item>"
   "<Title>"          -> create a title item
   "<Item>"           -> create a simple item
   "<CheckItem>"      -> create a check item
   "<ToggleItem>"     -> create a toggle item
   "<RadioItem>"      -> create a radio item
   <path>             -> path of a radio item to link against
   "<Separator>"      -> create a separator
   "<Branch>"         -> create an item to hold sub items (optional)
   "<LastBranch>"     -> create a right justified branch 
 */

static GtkItemFactoryEntry menu_items[] = {
        { "/_File",         NULL,         NULL, 0, "<Branch>" },
        { "/File/_New",    "<control>N", (GtkSignalFunc) menuCbNew, 0, NULL },
        { "/File/_Open",    "<control>O", (GtkSignalFunc) menuCbOpen, 0, NULL },
        { "/File/_Save",    "<control>S", (GtkSignalFunc) menuCbSave, 0, NULL },
        { "/File/Save _As", NULL,         (GtkSignalFunc) menuCbSaveAs, 0, NULL },
        { "/File/sep1",     NULL,         NULL, 0, "<Separator>" },
        { "/File/Quit",     "<control>Q", (GtkSignalFunc) cbQuit, 0, NULL },
        { "/_New",      NULL,         NULL, 0, "<Branch>" },
	{ "/Windows",	    NULL,	  NULL, 0, "<Branch>"},
        { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
        { "/_Help/About",   NULL,         (GtkSignalFunc) menuCbAbout, 0, NULL },
};

GtkItemFactory *setupMenu( GtkWidget  *window,
                GtkWidget **menubar )
{
        GtkItemFactory *item_factory;
        GtkAccelGroup *accel_group;
        gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

        accel_group = gtk_accel_group_new ();

        /* This function initializes the item factory.
           Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
           or GTK_TYPE_OPTION_MENU.
           Param 2: The path of the menu.
           Param 3: A pointer to a gtk_accel_group.  The item factory sets up
           the accelerator table while generating menus.
         */

        item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                        accel_group);

        /* This function generates the menu items. Pass the item factory,
           the number of items in the array, the array itself, and any
           callback data for the the menu items. */
        gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

        /* Attach the new accelerator group to the window. */
        gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

        if (menubar)
                /* Finally, return the actual menu bar created by the item factory. */
                *menubar = gtk_item_factory_get_widget (item_factory, "<main>");

	return item_factory;	
}

//! Creates the 'new' menu in the given GtkMenu.
void createNewMenu(GtkMenu *menu) {
	
	GtkWidget *effectItem = gtk_menu_item_new_with_label("Effect");
	gtk_menu_append(GTK_MENU(menu), effectItem);
	gtk_widget_show(effectItem);

	GtkWidget *effect = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(effectItem), effect);
	gtk_widget_show(effect);

	std::vector<FXGeneric *>::iterator it;

	for (it = glMainptr->vfx.begin(); it!=glMainptr->vfx.end(); ++it) {
		
		GtkWidget *item = gtk_menu_item_new_with_label((*it)->getName());
		gtk_menu_append(GTK_MENU(effect), item);
		gtk_signal_connect(GTK_OBJECT(item), "activate",
				GTK_SIGNAL_FUNC(GLMain::cbNewDuration), (*it)->getName());
		gtk_widget_show(item);
	}

	std::vector<DurationMaker *>::iterator dit;
	for (dit = glMainptr->durationFactory.makers.begin(); 
		dit != glMainptr->durationFactory.makers.end(); ++dit) {

		GtkWidget *item = gtk_menu_item_new_with_label((*dit)->description);

		gtk_signal_connect(GTK_OBJECT(item), "activate",
				GTK_SIGNAL_FUNC(GLMain::cbNewDuration), (*dit)->name);

		gtk_widget_show(item);

		if ((*dit)->type == DurationMaker::EFFECT) 
			gtk_menu_append(GTK_MENU(effect), item);
		else 
			gtk_menu_append(GTK_MENU(menu), item);
	}
}
